#!/usr/bin/env python

"""
iCal Calendar object.

Revision: $Rev: 165 $
Date: $Date: 2005-10-04 20:23:15 -0700 (Tue, 04 Oct 2005) $
Copyright: (c) 2005 Neil Jensen
License: MIT License. See LICENSE for details.

"""


from dateutil.tz import *
from datetime import *
from dateutil.parser import *
from dateutil import zoneinfo

from string import * 

from Cheetah.Template import Template

class Calendar:
	"iCal Calendar object"


	def __init__(self):
	
		self.freebusyDaysAhead = 60
		self.calscale = "GREGORIAN"  # default value
		self.organizer = ""
		self.url = ""
		
		# store the timezones as timezone types from the dateutils module
		self.timezones = ""
		
		# calendar client timezone; useful for some local time calculations
		self.clientTz = None
		
		# store calendar events in a list
		self.events = []
		
		# This is an array of freebusy arrays... each freebusy array will contain start/end tuples
		self.freebusyComponents = []

		# mandatory attributes:
		self.version = "2.0"
		self.prodid = "ifreebusy.com"


	# helper function
	def getTimezoneAwareDate(self, dateString, tzid):
	
		date = parse(dateString)
		
		if dateString[-1] != "Z":
			# use timezones defined in the iCal file
			if tzid and self.timezones:
				tz = self.timezones.get(tzid)
				return date.replace(tzinfo=tz)
					
			elif tzid:
				olsenTz = zoneinfo.gettz( tzid )
				return date.replace(tzinfo=olsenTz)
			
			# actually, this is probably a logic error and should not be done
			# use timezone defined by the calendar client
			#elif not tzid and self.clientTz:
			#	return date.replace(tzinfo=self.clientTz)
				
			# no timezones anywhere for us to use	
			elif not tzid:
				# we just guess and assign utc to this as no tz is specified
				return date.replace(tzinfo = tzutc() )	

		else:
			return date				
	
	
	def getFreeBusyArray(self, fromDate=datetime.now(tz=tzutc()), daysAhead=60):
		"""Return the sorted array of freebusy times for the specified time period"""
		
		endDate = fromDate + timedelta( days=daysAhead )			
		fbset = []	
		for event in self.events:
			evb = event.getFreeBusy(fromDate, daysAhead)
			if evb != []:
				fbset = fbset + evb
		
		fbset.sort()
		return fbset


	def getFreeBusyString(self, fromDate=datetime.now(tz=tzutc()), daysAhead=60):
		"""DEPRECATED - return complete freebusy string"""
		
		fbarray = self.getFreeBusyArray( fromDate, daysAhead )
		endDate = fromDate + timedelta( days=daysAhead )
		
		freebusyTemplateDef = """BEGIN:VCALENDAR
BEGIN:VFREEBUSY
#if $organizer
ORGANIZER:$organizer
#end if
#if $url
URL:$url
#end if
DTSTART:$datestart
DTEND:$dateend
#for $fbevent in $fbset
FREEBUSY:$fbevent
#end for
END:VFREEBUSY
END:VCALENDAR"""
				
		nameSpace = {'datestart': fromDate.astimezone(tzutc()).strftime("%Y%m%dT%H%M%SZ") , 
			     'dateend': endDate.astimezone(tzutc()).strftime("%Y%m%dT%H%M%SZ"), 
			     'url': self.url, 'organizer': self.organizer, 'fbset': fbarray }
		
		vfb = Template(freebusyTemplateDef, searchList=[nameSpace])					
		return vfb.respond()

	
	def getIcalStringFromFreebusy(self):
		"""Returns freebusy information converted into calendar format."""

		freebusyIcalTemplateDef = """BEGIN:VCALENDAR
PRODID:ifreebusy.com
VERSION:2.0
#set $uid = 0
#for $fbcomponent in $fbcomponents
#for $fbevent in $fbcomponent
#set $uid += 1
#if $fbevent
BEGIN:VEVENT
UID:$uid
#set $fb = $fbevent.split('/')
DTSTART:$fb[0]
DTEND:$fb[1]
SUMMARY:Busy
END:VEVENT
#end if
#end for
#end for
END:VCALENDAR"""
				
		nameSpace = {'prodid': self.prodid , 'version': self.version, 'fbcomponents': self.freebusyComponents }
		
		vfbical = Template(freebusyIcalTemplateDef, searchList=[nameSpace])
		return vfbical.respond()


	# Yet again, no attempt at being elegant, just go for brute force
	def getHcalStringFromIcal(self):
		"""returns calendar in hCalendar format 

		(see http://developers.technorati.com/wiki/hCalendar)
		"""

		freebusyHcalTemplateDef = """<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" 
"http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html>
<head>
<title>ifreebusy hCalendar</title> 
<link rel="stylesheet" type="text/css" href="/ifreebusyhcal.css" />
</head>
<body>
<span class="vcalendar">
#for $event in $events
<span class="vevent">
<span class="summary">$event.summary</span>
#set $dateStart = $event.dateStart.strftime("%Y%m%d")
<abbr class="dtstart" title="$dateStart">$dateStart</abbr>
#set $dateEnd = event.dateEnd.strftime("%Y%m%d")
<abbr class="dtend" title="$dateEnd">$dateEnd</abbr>
<span class="location">$event.location</span>
</span>
#end for
</span>
</body>
</html>"""
				
		nameSpace = {'events': self.events }
		
		vfbHcal = Template(freebusyHcalTemplateDef, searchList=[nameSpace])
		return vfbHcal.respond()


# end