#!/usr/bin/env python

"""
Class to parse iCal (RFC2445) files.

Revision: $Rev: 165 $
Date: $Date: 2005-10-04 20:23:15 -0700 (Tue, 04 Oct 2005) $
Copyright: (c) 2005 Neil Jensen
License: MIT License. See LICENSE for details.

"""

import sys
import re
from StringIO import StringIO
import logging

# dateutil module
import datetime
from dateutil.tz import *
from dateutil.parser import *
from dateutil import zoneinfo

import Calendar
import Event

class CalendarParser:
	"Parses iCal calendar data."
	
	logger = logging.getLogger('Calendar Parser')
	hdlr = logging.FileHandler('/dev/null')
	formatter = logging.Formatter('%(asctime)s %(levelname)s %(message)s')
	
	def __init__(self):		
		#set up logging for parsing
		self.hdlr.setFormatter(self.formatter)
		self.logger.addHandler(self.hdlr)
		self.logger.setLevel(logging.INFO)

		self.tzLines = []	# list to capture timezone related lines


	def parseFile(self, filename, debugURL=''):
		"""Parse file and returns Calendar objects. debugURL is so that Apache filter can pass URL for logging purposes."""	

		# get list of lines	from file or StringIO object	
		if filename.__class__.__name__ == "StringIO":
			lines = filename.readlines()
			self.logger.debug("opening stringIO object")
		else:
			lines = open(filename).readlines()
			self.logger.debug("------------------\nopening file " + filename)

		# flags used to maintain state during parsing
		inEvent = False
		inTimezone = False
		timezoneType = ""
		lines_index = 0
		durationString = ""
		inFreebusy = False

		for line in lines:
				
			# Deal with long lines that have been extended to the next line.
			# Look ahead and see if the next line(s) starts with a space, and if it does
			# add to the current line
			if lines_index < len(lines)-1:
			
				# check multiple lines ahead, not just one line ahead
				next_line_counter = 1
				next_line = lines[lines_index + next_line_counter]
				while (next_line[0] == " " or next_line == "\u0032" or next_line == "\u0009"):
					line = line.rstrip("\r\n") + next_line[1:]
					#self.logger.debug("line extension:" + line)
					next_line_counter += 1
					next_line = lines[lines_index + next_line_counter]
					
			lines_index = lines_index + 1

			if line[0] == " " or line == "\u0032" or line == "\u0009":
				continue
							
			# should be first line in a calendar file
			if line[0:15] == "BEGIN:VCALENDAR":
				self.logger.debug(line)
				self.ical = Calendar.Calendar()
				
			if line[0:12] == "X-WR-CALNAME":
				self.logger.debug(line)
		
			# last line in calendar, exit function
			elif line[0:13] == "END:VCALENDAR":
				self.logger.debug(line.rstrip("\r\n"))
				return self.ical
			
			# PRODID attribute... ignoring parameters for now
			elif line[0:6] == "PRODID":
				self.ical.prodid = line[line.index(":") + 1:].strip("\r\n")

			# VERSION attribute... ignoring parameters for now
			elif line[0:7] == "VERSION":
				self.ical.version = line[line.index(":") + 1:].strip("\r\n")
			
			# calendar scale property, usually GREGORIAN
			elif line[0:8] == "CALSCALE":
				# we don't do anything with these other than capture
				self.ical.calscale = line[9:].strip("\r\n")
				self.logger.debug("CALSCALE:" + self.ical.calscale)
			
			# indicates freebusy data
			elif line[0:15] == "BEGIN:VFREEBUSY":
				self.logger.debug(line)
				inFreebusy = True
				self.freebusyComponent = []

			# calendar organizer, typically used in freebusy
			elif line[0:9] == "ORGANIZER" and inFreebusy:
				# just capture this 
				self.ical.organizer = line[10:].strip("\r\n") 

			# freebusy elements
			# TODO: support parameters... this currently just looks at Outlook style data	
			elif line[0:8] == "FREEBUSY":
				self.freebusyComponent.append( line[line.index(":")+1:].strip("\r\n") )			


			# last of freebusy data, add the freebusyComponent to the array of freebusyComponents in the calendar
			elif line[0:13] == "END:VFREEBUSY":
				self.logger.debug(line)
				inFreebusy = False
				self.ical.freebusyComponents.append(self.freebusyComponent)	


			# X-WR-TIMEZONE;VALUE=TEXT:
			# Some calendars do not use VTIMEZONE components, but define this instead... 
			# I'm not sure what to do if we have both this and VTIMEZONE.
			# Evolution uses: X-LIC-LOCATION:
			# How about: it appears that this is the timezone of the client... let's use this for 
			# nailing down localtimes to UTC for freebusy purposes... i.e. if tz is not
			# defined for a dtstart, then apply clientTz to convert to UTC
			elif line[0:13] == "X-WR-TIMEZONE":
				clientTz = zoneinfo.gettz( line[line.index(":") + 1:].strip("\r\n") )
				self.ical.clientTz = clientTz
				self.logger.debug(line)
			
			elif line[0:14] == "X-LIC-LOCATION":				
				locale = line[line.index(":") + 1:].strip("\r\n")
				clientTz = zoneinfo.gettz( locale )

				self.ical.clientTz = clientTz
				self.logger.debug(line)						
					
			# we are in the timezone component, use tz type from dateutil module
			elif line[0:15] == "BEGIN:VTIMEZONE":
				inTimezone = True
				# create a list to capture the tz lines
				self.tzLines.append(line)
				self.logger.debug(line)
				
			elif line[0:13] == "END:VTIMEZONE":
				self.tzLines.append(line)
				tzfilelines = StringIO( ''.join(self.tzLines) )
				inTimezone = False
				
				# add the timezone type to the calendar (not particularly 
				# efficient; we do it as many times as there are tz's, but it works) 
				self.logger.debug( self.tzLines )
				try:
					vtimezones = tzical(tzfilelines)
					self.ical.timezones = vtimezones
					self.logger.debug( vtimezones.keys() )
				# if we can't parse the vtimezones lines properly, trap and use UTC	
				except ValueError:
					self.logger.debug("Error parsing tzlines")
					sys.stderr.write(str(datetime.datetime.now().isoformat()) + " - lines ending:" + str(lines_index) + " Error creating timezones from tzlines for URL:" + debugURL )
					sys.stderr.flush() 
					pass
				
			
			# catch all of the lines within the timezone component,
			# except for xparams starting with X-, or with UID (Entourage problem) and LOCATION (Mozcal)
			elif line[0] != "X" and line[0:3] != "UID"  and line[0:8] != "LOCATION" and inTimezone:
				#self.logger.debug(inTimezone)
				self.tzLines.append( line )
			
			# RFC2445 4.6.1 - begin VEvent   
			elif line[0:12] == "BEGIN:VEVENT":    
				inEvent = True
				self.logger.debug("\n\n--------\n" + line + " ") 
				event = Event.Event(self.ical)
			
			# RFC2445 - 4.8.2.4 - DTSTART property
			elif line[0:7] == "DTSTART" and inEvent:
				# default type is DATE-TIME, but can be DATE for reminder/anniversary
				# time value must conform to DATE-TIME value type
				# Parameters allowed: non-standard, tzid, value data type
				self.logger.debug(line)
				
				# We convert all times to UTC... either based on tzid in the DTSTART line or in the 
				# clientTZ timezone if no tzid defined in DTSTART for local times. Failing that, we default to UTC
				m = re.compile("^DTSTART(;VALUE=.*)?(;TZID=(.*))?(;.*)*:(.*)").match(line)
				
				# use dateutil module to parse date(-time) and create dt type, 'Z' at end, automatically in UTC				
				dateString = m.group(5)
				tzid = m.group(3)
				self.logger.debug(dateString)
				if tzid:
					self.logger.debug("DSTART timezone:" + tzid)
				event.dateStart = self.ical.getTimezoneAwareDate(dateString, tzid)	
				
			elif line[0:5] == "DTEND" and inEvent:
				self.logger.debug(line)
				m = re.compile("^DTEND(;VALUE=.*)?(;TZID=(.*))?(;.*)*:(.*)").match(line)
				dateString = m.group(5)
				tzid = m.group(3)
				event.dateEnd = self.ical.getTimezoneAwareDate(dateString, tzid)
				
			
			elif line[0:8] == "DURATION" and inEvent:
				m = re.compile("^DURATION(;.*)*(;.*\".*:.*\")*:(.*)").match(line)
				durationString = m.group(3)
				self.logger.debug("Duration=" + durationString)
						
				
			# RFC2445 4.8.5.4 - RRULE	
			elif line[0:5] == "RRULE" and inEvent:
				# Event type stores rrule as a string
				m = re.compile("^RRULE(;.*)*(;.*\".*:.*\")*:(.*)").match(line)
				rule = m.group(3)
				self.logger.debug("RRULE added:" + rule)
				event.rrules.append("RRULE:" + rule + "\n")
				
			# RFC2445 4.8.5.2 - EXRULE	
			elif line[0:6] == "EXRULE" and inEvent:
				# Event type stores rrule as a string
				m = re.compile("^EXRULE(;.*)*(;.*\".*:.*\")*:(.*)").match(line)
				exrule = m.group(3)
				self.logger.debug("EXRULE added:" + exrule)
				event.exrules.append("EXRULE:"+ exrule + "\n")
	
			# RFC2445 4.8.5.3 - RDATE	
			elif line[0:5] == "RDATE" and inEvent:
				# rdate type stores entire line because params are important
				self.logger.debug(line)
				event.rdates.append(line)
				
			# RFC2445 4.8.5.1 - EXDATE	
			elif line[0:6] == "EXDATE" and inEvent:
				# exdate type stores entire line because params are important
				self.logger.debug(line)
				event.exdates.append(line)
	
	
			elif line[0:7] == "SUMMARY" and inEvent:
				m = re.compile("^SUMMARY(;.*)*(;.*\".*:.*\")*:(.*)").match(line)
				event.summary = m.group(3)
				self.logger.debug(m.group(3).strip("\r\n"))

			elif line[0:7] == "TRANSP:" and inEvent:
				event.setTransparency(line[7:].strip("\r\n"))

			elif line[0:8] == "LOCATION" and inEvent:
				m = re.compile("^LOCATION(;.*)*(;.*\".*:.*\")*:(.*)").match(line)
				event.location = m.group(3) 


			elif line[0:10] == "END:VEVENT":
				# call the setDuration method now to ensure that we have the startDate already
				event.setDuration(durationString)
				self.ical.events.append(event)
				event = None
				durationString = ""
				self.logger.debug("\n^^^" + line + " ")
				inEvent = False


		

	
