#!/usr/bin/env python

"""
iCal Calendar Event component.

Revision: $Rev: 164 $
Date: $Date: 2005-10-04 20:12:28 -0700 (Tue, 04 Oct 2005) $
Copyright: (c) 2005 Neil Jensen
License: MIT License. See LICENSE for details.

"""
import sys
from dateutil.tz import *
from dateutil.rrule import *
from dateutil.parser import *
from datetime import *

import string
from StringIO import StringIO
import re

class Event:
	"iCal Calendar Event component"
		
	def __init__(self, calendar):
		# Event properties that we care about
		self.summary = ""
		self.duration = ""
		self.location = ""
		self.isTransparent = False
		self.calendar = calendar # keep reference to the calendar we belong to
		self.freebusy = []
	
		# simply stored as rule strings, excluding parameters
		self.rrules = []
		self.exrules = []
	
		# stored as strings, complete including rparams as they are important
		self.rdates = []
		self.rdatesFreebusyWithDifferentDurations = []	# ugly hack
		self.exdates = []
	
		# Store all dates in UTC time, tz values currently set by parser
		self.dateStart = None
		self.dateEnd = None

	def setTransparency(self, eventTransparency):
		if eventTransparency == "TRANSPARENT":
			self.isTransparent = True

	def setDuration(self, durationString):
		self.duration = durationString
		
		# we'll create a dateEnd if it is not defined and we have a duration
		if self.dateStart and not self.dateEnd and self.duration:	
			self.dateEnd = self.getDateEndFromDuration(self.dateStart, durationString)
		
		
	def getDateEndFromDuration(self, eventDateStart, durationString):
		match = re.compile("^([+-])?P(\d+W)?(\d+D)?T?(\d+H)?(\d+M)?(\d+S)?").match(durationString)
				
		[w,d,h,m,s] = [0,0,0,0,0]
				
		if match.group(2):
			w = int(match.group(2)[0:-1])
		if match.group(3):
			d = int(match.group(3)[0:-1])
		if match.group(4):
			h = int(match.group(4)[0:-1])
		if match.group(5):
			m = int(match.group(5)[0:-1])
		if match.group(6):
			s = int(match.group(6)[0:-1])

		td = timedelta(weeks=w,days=d,hours=h,minutes=m,seconds=s)
		dateEnd = eventDateStart + td
				
		# put dateEnd in same timezone as dateStart
		startTz = eventDateStart.tzinfo
		if startTz:
			dateEnd = dateEnd.replace(tzinfo=startTz)

		return dateEnd

				
	# WARNING: this is only returning recurrence sets that have instances that will show up in freebusy
	def getRecurrenceSet(self, fromDate=datetime.now(tz=tzutc()), daysAhead = 60):

		rrset = None

		# get events that occur between now and the number of days defined in the calendar object
		endRecurrenceDate = fromDate + timedelta( days=daysAhead )
	
		# return a dateutil.rrule set with complete set of recurrences for this event (tz aware)
		rrstring = "".join(self.rrules + self.exrules)
		
		# account for non-repeating events by just using dateStart
		if rrstring == "":
			rrset = rrule(DAILY, count=1, dtstart=self.dateStart)			
		else:
			# parse the rulesets, also send a default tz just in case we have problems with unknown tz's
			rrset = rrulestr(rrstring, forceset=True, compatible=True, dtstart=self.dateStart )
			
		# remove a datetime, must also include timezone
		for exdate in self.exdates:
			m = re.compile("^EXDATE(;VALUE=.*)?(;TZID=(.*))?(;.*)*:(.*)").match(exdate)	
			tzid = m.group(3)
			
			# look for multiple instances of datetimes 
			dtlist = re.compile("([\w]+)+").findall( m.group(5).strip("\r\n") ) 
			for dateString in dtlist:
				rrset.exdate( self.calendar.getTimezoneAwareDate(dateString, tzid) )

		# TODO: add handling of rdates 
		for rdate in self.rdates:
			m = re.compile("^RDATE(;VALUE=([\w]+))?(;TZID=(.*))?(;.*)*:(.*)").match(rdate)	
			tzid = m.group(4)
			rdates = m.group(6)
			
			if m.group(2) == "DATE":
				# Ignore RDATE's that are DATE type, as they don't get added to freebusy info
				# Just for now... this is still part of the re-occurence, but does not show as freebusy
				continue
			
			# Handle the rdates that have different durations than the VEVENT.
			# This is a very ugly hack... because these events may have different durations
			# than the VEVENT, add them to a separate list rdatesFreebusyWithDifferentDurations
			# for later merging with the overall list of freebusy events. 
			elif m.group(2) == "PERIOD" or (string.find(rdates, "/") > 0):
				#look for multiple instances
				dtlist = re.compile("[\w]+/[\w]+").findall( rdates ) 
				for dateString in dtlist:
					[startDatetime, endDatetime] = string.split(dateString,"/") 
					startDatetime = self.calendar.getTimezoneAwareDate(startDatetime, tzid)
					if endDatetime[0] == "P":
						endDatetime = self.getDateEndFromDuration(startDatetime, endDatetime)
						
					else: 
						endDatetime = self.calendar.getTimezoneAwareDate(endDatetime, tzid)
					
					# Add this to the rdatesFreebusyWithDifferentDurations in right format 
					# (only if it is in the window of time we are looking at).
					# Keep it simple for now and only check that startDatetime is in the window,
					# which is how the dateutil module works.
					if (fromDate < startDatetime < endRecurrenceDate):		
						tz = tzutc()
						startfb = startDatetime.astimezone(tz).strftime("%Y%m%dT%H%M%SZ")
						endfb = endDatetime.astimezone(tz).strftime("%Y%m%dT%H%M%SZ")				
						self.rdatesFreebusyWithDifferentDurations.append( startfb + "/" + endfb )	

				#print self.rdatesFreebusyWithDifferentDurations
			
			else:
				# look for multiple instances of datetimes 
				dtlist = re.compile("([\w]+)").findall( rdates ) 
				for dateString in dtlist:
					rrset.rdate( self.calendar.getTimezoneAwareDate(dateString, tzid) )

		# The rrule module does not handle UNTIL rules that are date type, only datetime
		# we'll trap these errors only
		try:
			return rrset.between( fromDate, endRecurrenceDate, inc=True)
		except (TypeError), why:
			if str(why) == "can't compare offset-naive and offset-aware datetimes":
				#print "\n" + self.summary + "\n" + str(why) + "\nRRULE:" + rrstring
				return []
			else:
				raise
	
	# generate list of freebusy datetime strings	
	def getFreeBusy(self, fromDate=datetime.now(tz=tzutc()), daysAhead=60):
		self.freebusy=[]

		# don't show this as busy if the event has no duration or is transparent
		if self.isTransparent == True or not self.dateEnd:
			return []
		else:
			try:	
				duration =  self.dateEnd - self.dateStart 	

				tz = tzutc()
			
				rrset = self.getRecurrenceSet(fromDate, daysAhead)	
				for occurence in list(rrset):
					startfb = occurence.astimezone(tz).strftime("%Y%m%dT%H%M%SZ")
					endfb = (occurence + duration).astimezone(tz).strftime("%Y%m%dT%H%M%SZ")
				
					self.freebusy.append( startfb + "/" + endfb )
			
				# now add the recurrences with different durations to the freebusy list
				self.freebusy = self.freebusy + self.rdatesFreebusyWithDifferentDurations
				return self.freebusy

			except:
				info  = sys.exc_info() 
				raise FreeBusyException( self.summary , self.dateStart, self.dateEnd, info ) 		
					
				

class FreeBusyException(Exception):
	
	def __init__(self, eventSummary, eventDateStart, eventDateEnd, info):
		self.eventSummary = eventSummary 
		self.eventDateStart = eventDateStart
		self.eventDateEnd = eventDateEnd
		self.info = info 


	def __str__(self):
		errorMessage = ""
		if self.eventSummary:  	
			errorMessage = errorMessage +  "\nSummary:" + self.eventSummary	
		if self.eventDateStart: 
			errorMessage = errorMessage +  " Start date:" + repr(self.eventDateStart) 
		if self.eventDateEnd:
			errorMessage  = errorMessage +  " End date:" + repr(self.eventDateEnd)    

		errorMessage = errorMessage + "\nline:" + repr(self.info[2].tb_lineno) + ", " + str(self.info[0]) + ", " + str(self.info[1]) 
	
		return errorMessage



				
				
